// tfam_mouse.c

#include "config.h"
#include "lib_mcu/usb/usb_drv.h"
#include "usb_descriptors.h"
#include "modules/usb/device_chap9/usb_standard_request.h"
#include "tfam_led.h"

#define MOUSE_SPEED 4
#define DEBOUNCE_MS 10

// OLD hardware (breadboard):
// D3/INT3: Y1
// D4/INT5: X1
// B0: Y2
// B1: BUTTON
// B2: X2

// NEW hardware (PCB):
// D1: X2
// D2/INT2: X1
// D3/INT3: Y1
// D4: Y2
// D5: BUTTON

static volatile S8 mouse_x;
static volatile S8 mouse_y;
static volatile bit mouse_button;
static volatile U8 debounce_count;

static S8 sent_x;
static S8 sent_y;
static bit sent_button;

// TODO: i'd rather do this using a real timer
void sof_action() {
    bit button;
    
    button = (PIND & 0x20) ?  FALSE : TRUE;
    if (button == mouse_button) {
        if (debounce_count != 0) {
            debounce_count--;
        }
    } else {
        mouse_button = button;
        debounce_count = DEBOUNCE_MS;
    }
    tfam_led_tick();
}

void tfam_mouse_task() {
    S8 x, y;
    bit button;

    if (! Is_device_enumerated()) {
        return;
    }
    Usb_select_endpoint(EP_MOUSE_IN);
    if (! Is_usb_in_ready()) {
        return;
    }

    x = mouse_x;
    y = mouse_y;
    button = mouse_button;
    if (debounce_count != 0) {
        button = sent_button;
    }
	if ((x == sent_x) && (y == sent_y) && (button == sent_button)) {
	    return;
	}

    Usb_write_byte(button ? 1 : 0);
    Usb_write_byte(x - sent_x);
    Usb_write_byte(y - sent_y);
    Usb_write_byte(0);
    Usb_ack_in_ready(); // tell the USB controller to flush the buffer

    sent_x = x;
    sent_y = y;
    sent_button = button;
}

void tfam_mouse_task_init() {
    // disable interrupts
    EIMSK &= ~((1 << INT2) | (1 << INT3));
    // clear state variables
    mouse_x = 0;
    mouse_y = 0;
    mouse_button = FALSE;
    debounce_count = 0;
    sent_x = 0;
    sent_y = 0;
    sent_button = FALSE;
    tfam_led_init();
    // set all 5 pins to input mode and enable pullup resistors
    DDRD &= ~0x3e;
    PORTD |= 0x3e;
    // configure interrupts
    EICRA |= (1 << ISC20) | (1 << ISC30);
    // enable interrupts
    EIMSK |= (1 << INT2) | (1 << INT3);
    Usb_enable_sof_interrupt();
    sei();
}

ISR(INT3_vect) {
    bit y1, y2;
    y1 = ((PIND & 0x08) ?  TRUE : FALSE);   
    y2 = ((PIND & 0x10) ?  TRUE : FALSE);
    if (y1 ^ y2) {
        mouse_y += MOUSE_SPEED;
    } else {
        mouse_y -= MOUSE_SPEED;
    }
    tfam_led_light();
}

ISR(INT2_vect) {
    bit x1, x2;
    x1 = ((PIND & 0x04) ?  TRUE : FALSE);   
    x2 = ((PIND & 0x02) ?  TRUE : FALSE);
    if (x1 ^ x2) {
        mouse_x -= MOUSE_SPEED;
    } else {
        mouse_x += MOUSE_SPEED;
    }
    tfam_led_light();
}
